# Vehicle class

import sys, math

import pygame
from pygame.locals import *

import tools, parameters

class Vehicle(pygame.sprite.Sprite):
    """Vehicle class"""
    VELOCITY = 3
    INITIAL_DIRECTION = 15

    turn_delay = 0
    orig_rect = None
    orig_direction = None
    #right_sensor = Sensor.Sensor()
    
    def __init__(self, position, direction):
        pygame.sprite.Sprite.__init__(self)
        (self.image, self.rect) = tools.load_image('bv.png', (0,0))

        self.orig_direction = direction

        self.INITIAL_DIRECTION = direction

        self.rect.center = position 

        self.direction = 0

        self.original = self.image

        self.state = 'stop'

        self.orig_rect = Rect(self.rect)

        self.rotate(self.INITIAL_DIRECTION)
        #self.right_sensor.rotate(self.INITIAL_DIRECTION)

    def update(self):

        if self.state == 'drive':
            self.drive()
        elif self.state == 'stop':
            pass

    def reset(self):
        self.rect.center = self.orig_rect.center
        self.rotate(self.orig_direction - self.direction)
        self.state = 'stop'
        
    def get_col_rect(self):
        # tighter rect for collisions
        col_rect = Rect(self.rect)
        col_rect.inflate_ip(-15,-13)
        return col_rect

    def get_direction(self):
        return self.direction * 22.5
        
    def drive(self):

        vel = self.VELOCITY
        dir = self.direction

        x, y = tools.get_x_y(dir, vel)
        
        newpos = self.rect.move((x, y))

        self.rect = newpos
        

    def rotate(self, delta_deg):
        center = self.rect.center

        self.direction = (self.direction + delta_deg) % 16
        
        self.image = pygame.transform.rotate(self.original, self.direction * 22.5)

        self.rect = self.image.get_rect()
        self.rect.center = center

    def turn_left(self):
        self.rotate(1)
        #self.right_sensor.rotate(1)
    def turn_right(self):
        self.rotate(-1)
    def stop(self):
        self.state = 'stop'

    def chk_prox(self, lamps):
        if self.state != 'drive': return
        
        for lamp in lamps:
            
            if lamp.state == 'dropped':
                
                self.lamp = lamp
                # check if lamp is in range of sensors
                dist = tools.mag(self.rect.center, lamp.rect.center) 
                #if dist < parameters.SENSOR_RANGE and self.turn_delay > parameters.TURN_THRESH:
                if dist < parameters.SENSOR_RANGE:
                   #print 'in range'
                   right = self.chk_right_sensor()
                   left = self.chk_left_sensor()
                   
                   if right:
                       self.turn_left()
                   elif left:
                       self.turn_right()
                       self.turn_delay = 0
                   else:
                       self.turn_delay = self.turn_delay+1
                    
            

    def chk_right_sensor(self):
        return self.chk_sensor('r')
        
    def chk_left_sensor(self):
        return self.chk_sensor('l')

    def chk_sensor(self, sensor):

        if sensor == 'l': offset = 90
        elif sensor == 'r': offset = -90
        else:
            print 'Error sensor: '+sensor+' not known.'
            sys.exit()
            
        dir_s= self.get_direction() * -1 + 180

        dir_d = (self.get_direction() + offset) % 360 * -1 + 180
                
        vec_s = (parameters.SENSOR_RANGE * math.cos(math.radians(dir_s)),\
                      parameters.SENSOR_RANGE * math.sin(math.radians(dir_s)))

        vec_d=(parameters.SENSOR_DEPTH * math.cos(math.radians(dir_d)),\
                    parameters.SENSOR_DEPTH * math.sin(math.radians(dir_d)))

        point_a = self.rect.center # vehcile pos

        point_b = tools.vec_add(point_a, vec_s) #sensor range pos

        point_c = tools.vec_add(point_b,vec_d) # sensor depth pos

        point_p = self.lamp.rect.center # lamp position

        return tools.tri_contains(point_a, point_b, point_c, point_p)
        
            
